package at.ac.uibk.gitsearch.es.model;

import java.util.List;
import java.util.Objects;

import com.fasterxml.jackson.annotation.JsonEnumDefaultValue;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonValue;

import at.ac.uibk.gitsearch.service.dto.MetadataUserDTO;

@JsonInclude(JsonInclude.Include.NON_NULL)
public class ExerciseInfo {

    private String metadataVersion;

    private ExerciseType type;

    private String title;

    private String license;

    private String description;

    private List<String> keyword;

    private List<String> format;

    private String structure;

    private List<String> programmingLanguage;

    private List<String> language;

    private ExerciseDifficulty difficulty;

    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private List<MetadataUserDTO> creator;

    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private List<MetadataUserDTO> contributor;

    private List<String> source;

    public ExerciseInfo() {
    }

    public String getMetadataVersion() {
        return metadataVersion;
    }

    public void setMetadataVersion(String metadataVersion) {
        this.metadataVersion = metadataVersion;
    }

    public ExerciseType getType() {
        return type;
    }

    public void setType(ExerciseType type) {
        this.type = type;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getLicense() {
        return license;
    }

    public void setLicense(String license) {
        this.license = license;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public List<String> getKeyword() {
        return keyword;
    }

    public void setKeyword(List<String> keyword) {
        this.keyword = keyword;
    }

    public List<String> getFormat() {
        return format;
    }

    public void setFormat(List<String> format) {
        this.format = format;
    }

    public String getStructure() {
        return structure;
    }

    public void setStructure(String structure) {
        this.structure = structure;
    }

    public List<String> getProgrammingLanguage() {
        return programmingLanguage;
    }

    public void setProgrammingLanguage(List<String> programmingLanguage) {
        this.programmingLanguage = programmingLanguage;
    }

    public List<String> getLanguage() {
        return language;
    }

    public void setLanguage(List<String> language) {
        this.language = language;
    }

    public ExerciseDifficulty getDifficulty() {
        return difficulty;
    }

    public void setDifficulty(ExerciseDifficulty difficulty) {
        this.difficulty = difficulty;
    }

    public List<String> getSource() {
        return source;
    }

    public void setSource(List<String> source) {
        this.source = source;
    }

    @JsonProperty("creator")
    public List<MetadataUserDTO> getCreator() {
        return creator;
    }

    public void setCreator(List<MetadataUserDTO> creator) {
        this.creator = creator;
    }

    @JsonProperty("contributor")
    public List<MetadataUserDTO> getContributor() {
        return contributor;
    }

    public void setContributor(List<MetadataUserDTO> contributor) {
        this.contributor = contributor;
    }

    @Override
    public String toString() {
        return "ExerciseInfo{" +
            "metadataVersion='" + metadataVersion + '\'' +
            ", type=" + type +
            ", title='" + title + '\'' +
            ", license='" + license + '\'' +
            ", keyword=" + keyword +
            ", format=" + format +
            ", structure='" + structure + '\'' +
            ", programmingLanguage=" + programmingLanguage +
            ", language=" + language +
            ", difficulty=" + difficulty +
            ", creator=" + creator +
            ", contributor=" + contributor +
            ", source=" + source +
            '}';
    }

    @JsonFormat(with = JsonFormat.Feature.ACCEPT_CASE_INSENSITIVE_PROPERTIES)
    public enum ExerciseType {PROGRAMMING_EXERCISE("programming exercise"), EXERCISE("exercise"), COLLECTION("collection"), OTHER("other");
        ExerciseType(String s) {
        }

        @JsonValue
        public String jacksonCustomMapping() {
            return name().toLowerCase().replace("_", " ");
        }

        @Override
        public String toString() {
            return name().toLowerCase().replace("_", " ");
        }
    }

    @JsonFormat(with = JsonFormat.Feature.ACCEPT_CASE_INSENSITIVE_PROPERTIES)
    public enum ExerciseDifficulty {
        SIMPLE("simple"),
        MEDIUM("medium"),
        ADVANCED("advanced"),
        @JsonEnumDefaultValue NONE("none");

        ExerciseDifficulty(String s) {}

        @JsonValue
        public String jacksonCustomMapping() {
            return name().toLowerCase();
        }

        @Override
        public String toString() {
            return name().toLowerCase();
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        ExerciseInfo that = (ExerciseInfo) o;
        return Objects.equals(getMetadataVersion(), that.getMetadataVersion()) && getType() == that.getType() && Objects.equals(getTitle(), that.getTitle()) && Objects.equals(getLicense(), that.getLicense()) && Objects.equals(getDescription(), that.getDescription()) && Objects.equals(getKeyword(), that.getKeyword()) && Objects.equals(getFormat(), that.getFormat()) && Objects.equals(getStructure(), that.getStructure()) && Objects.equals(getProgrammingLanguage(), that.getProgrammingLanguage()) && Objects.equals(getLanguage(), that.getLanguage()) && getDifficulty() == that.getDifficulty() && Objects.equals(getCreator(), that.getCreator()) && Objects.equals(getContributor(), that.getContributor()) && Objects.equals(getSource(), that.getSource());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getMetadataVersion(), getType(), getTitle(), getLicense(), getDescription(), getKeyword(), getFormat(), getStructure(), getProgrammingLanguage(), getLanguage(), getDifficulty(), getCreator(), getContributor(), getSource());
    }
}
